
import org.json.*;
import java.net.URLEncoder;

// import UDP library
import hypermedia.net.*;

int size = 60;       // Width of the shape
float xpos, ypos;    // Starting position of shape    

float xspeed = 3.8;  // Speed of the shape
float yspeed = 3.2;  // Speed of the shape

int xdirection = 1;  // Left or Right
int ydirection = 1;  // Top to Bottom

// Initial color
int r = 255;
int g = 255;
int b = 255;

boolean bit1_val = false;
boolean bit2_val = false;
boolean bit3_val = false;
boolean bit4_val = false;

UDP udp;  // define the UDP object

// ------------------------------------------------------------------
// DeviceServer設定
// 
// * サーバーホスト URL 設定
//   DeviceServer 自身の HTTP サーバーを使用する場合には空にする
//
//String server_host_url = "http://your_DeviceServer_public_url:80";
String server_host_url = "http://localhost:8080";
//String server_host_url = "";
//
// * SessionToken 認証を省略して既存のセッショントークンを使用して
//   DeviceServer にアクセスする
//
String session_token = "1234";
//

// ------------------------------------------------------------------
// DeviceServer のスクリプトを起動する (WebAPI 経由でリクエストを発行する)
// スクリプトへのリクエストパラメータを指定して、スクリプト中で設定されたリプライデータを
// 受信する
boolean ExecuteScriptWebAPI(String scriptName,HashMap reqParams,HashMap rplParams){
    try {  
        String url = server_host_url + "/command/json/script?name=" + scriptName + "&session=" + session_token;
        // construct the request url with parameters
        Iterator i = reqParams.entrySet().iterator();  // Get an iterator
        while (i.hasNext()) {
            Map.Entry me = (Map.Entry)i.next();
            url = url + "&" + URLEncoder.encode((String)me.getKey(),"UTF-8") + "=" + URLEncoder.encode((String)me.getValue(),"UTF-8");
        }
        // execute the script on DeviceServer
        String result = join(loadStrings(url), "");  
        // parse reply JSON and construct a reply HashMap
        JSONObject result_all = new JSONObject(result);
        if(result_all.getString("Result").equals("Success")){
            JSONObject result_params = result_all.getJSONObject("ResultParams");
            for (Iterator j = result_params.keys(); j.hasNext();) {
              String key = (String)j.next();
              String val = result_params.getString(key);
              rplParams.put(key, val);
            }
        } else {
            throw (new Exception((String)result_all.getString("ErrorText")));
        }
        return true;
    } catch(Exception e) {
        println("ExecuteScript:*EXCPTION* " + e.getMessage()); 
        return false;
    }
}

// DeviceServer の "PUT_IO" スクリプトを起動して UIOUSB デバイスの
// I/O ポート値を変更する
void update_io(int bit,boolean val){
    HashMap req = new HashMap();
    HashMap rpl = new HashMap();
    req.put("BIT",  str(bit));
    req.put("VAL", val ? "1" : "0");  
    ExecuteScriptWebAPI("PUT_IO",req,rpl);
}

void setup() 
{
  size(400, 400);
  noStroke();
  frameRate(60);
  smooth();
  // Set the starting position of the shape
  xpos = width/2;
  ypos = height/2;
  
  udp = new UDP( this, 8091 );
  //udp.log( true ); 		// <-- printout the connection activity
  udp.listen( true );

  // AD0 の値を取得して、shape の初期サイズに設定する
  HashMap req = new HashMap();
  HashMap rpl = new HashMap();
  ExecuteScriptWebAPI("GET_AD0",req,rpl);
  size = int((String)rpl.get("AD0"));
}




void draw() 
{
  background(102);
  
  // Update the position of the shape
  xpos = xpos + ( xspeed * xdirection );
  ypos = ypos + ( yspeed * ydirection );
  
  // Test to see if the shape exceeds the boundaries of the screen
  // If it does, reverse its direction by multiplying by -1
  if (xpos > width-size) {
    xdirection *= -1;
    bit1_val = !bit1_val;
    update_io(1,bit1_val);
  }

  if (xpos < 0) {
    xdirection *= -1;
    bit2_val = !bit2_val;
    update_io(2,bit2_val);
  }

  if (ypos > height-size ) {
    ydirection *= -1;
    bit3_val = !bit3_val;
    update_io(3,bit3_val);
  }

  if (ypos < 0) {
    ydirection *= -1;
    bit4_val = !bit4_val;
    update_io(4,bit4_val);
  }

  // Draw the shape
  fill(r,g,b);
  ellipse(xpos+size/2, ypos+size/2, size, size);
}

void receive( byte[] data, String ip, int port ) {	// <-- extended handler
  String message = new String( data );
  
  // "AD" で始まる AD0 変換値を受信した場合は shape のサイズをAD0 の値に基づいて変更する
  if (message.substring(0,2).equals("AD")) {
    size = int(message.substring(2)) / 2;
  } 
  // "SW" で始まるスイッチ入力を受信した場合には shape の色をランダムに変更する
  if (message.substring(0,2).equals("SW")) {
    r = int(random(255));
    g = int(random(255));
    b = int(random(255));
  } 
}


 

